from .crud import KeycloakCRUD
from .targets import Targets
from .groups import Groups
from .roles import RolesURLBuilder
from .users import Users
from .realms import Realms
from .url import RestURL
from .idp import IdentityProviderURLBuilder
from .auth_flows import AuthenticationFlows, AuthenticationFlowURLBuilder
from .clients import Clients

KCResourceTypes = {
    "users": Users,
    "groups": Groups,
    "realms": Realms,
    "authentication": AuthenticationFlows,
    "clients": Clients
}

URLBuilders = {
    'roles': RolesURLBuilder,
    'authentication': AuthenticationFlowURLBuilder,
    "idp": IdentityProviderURLBuilder,
    "identity-provider": IdentityProviderURLBuilder
}


def GenericURLBuilder(url):
    targets = Targets.makeWithURL(url)
    return targets


class KCResourceBuilder:
    def __URLSetup(self, url):
        return RestURL(url=url, resources=["auth", "admin", "realms"])

    def __init__(self, keycloakURL):
        self.name = None
        self.realm = None
        self.url = self.__URLSetup(keycloakURL)

    def withName(self, name):
        self.name = name
        return self

    def forRealm(self, realm):
        self.realm = realm
        return self

    def build(self, token):
        KCResourceAPI = KeycloakCRUD if not self.name in KCResourceTypes else KCResourceTypes[self.name]
        URLBuilder = GenericURLBuilder if not self.name in URLBuilders else URLBuilders[self.name]

        self.url.addResources([self.realm, self.name])

        resource = KCResourceAPI()
        resource.targets = URLBuilder(str(self.url))
        resource.token = token

        return resource



import unittest, time
from kcapi import OpenID, RestURL
from .testbed import TestBed
import json

class Testing_Realm_API(unittest.TestCase):

    def testing_realm_api_methods(self):
        realms = self.testbed.getKeycloak().build('realms', self.REALM)
        self.assertTrue(hasattr(realms, 'caches'))


    def testing_realm_cache_reset(self):
        realms = self.testbed.getKeycloak().build('realms', self.REALM)

        caches = realms.caches(self.REALM)

        self.assertEqual(caches.clearRealmCache().resp().status_code, 204)

    def testing_user_cache_reset(self):
        realms = self.testbed.getKeycloak().build('realms', self.REALM)

        caches = realms.caches(self.REALM)

        self.assertEqual(caches.clearUserCache().resp().status_code, 204)


    def testing_key_cache_reset(self):
        realms = self.testbed.getKeycloak().build('realms', self.REALM)

        caches = realms.caches(self.REALM)

        self.assertEqual(caches.clearKeyCache().resp().status_code, 204)


    @classmethod
    def setUpClass(self):
        self.testbed = TestBed()
        self.testbed.createRealms()
        self.testbed.createUsers()
        self.testbed.createClients()
        self.REALM = self.testbed.REALM

    @classmethod
    def tearDownClass(self):
        self.testbed.goodBye()

if __name__ == '__main__':
    unittest.main()



import unittest, time
from rhsso import OpenID, KeycloakAdmin, discover, RestURL

sso = OpenID({
        "client_id": "admin-cli",
        "username": "admin", 
        "password":"admin1234", 
        "grant_type":"password",
        "realm" : "master"
        })

test_url = "https://sso-cvaldezr-dev.apps.sandbox.x8i5.p1.openshiftapps.com"

class TestingSSOAPI(unittest.TestCase):

    def testing_discovery_url(self):
        json_resp = discover(test_url, "master")
        self.assertNotEqual(json_resp, None)

     
    def test_login(self):
        token = sso.getToken(test_url)
        self.assertNotEqual(token, None)

    def testing_passing_non_openid_url(self):
        self.assertRaises(Exception, lambda: sso.getToken("https://www.google.com")) 

    def testing_CRUD_api(self):
        admin = self.admin

        realm = admin.master()
        self.assertNotEqual(realm.getHeaders(), None)

        state = realm.create({"enabled": "true", "id": "my_realm_1", "realm": "my_realm_1"})
        self.assertEqual(state, True)

        obj = realm.findById("my_realm_1")
        self.assertEqual(obj['id'], 'my_realm_1')

        obj['displayName'] = "MyRealm"
        state_update = realm.update("my_realm_1", obj)
        self.assertEqual(state_update, True)

        updated_object = realm.findById("my_realm_1")
        self.assertEqual(updated_object['displayName'], "MyRealm")

        remove_state = realm.remove("my_realm_1")
        self.assertEqual(remove_state, True)


    def testing_building_your_own_URL(self):
       my_url =  RestURL("https://sso-cvaldezr-dev.apps.sandbox.x8i5.p1.openshiftapps.com/auth/admin")
       my_url.addResources(['realms'])
       realms = self.admin.buildForURL(my_url)
       realms_list = realms.findAll()

       self.assertEqual(realms_list.status_code, 200)
       self.assertGreater(len(realms_list.json()), 1)

    def testing_graceful_handling_of_incorrect_credentials(self):
       kc = KeycloakAdmin({
            "username": "bad_user", 
            "password": "bad_password"
            }, test_url)

       realm = admin.master()

       state = realm.create({"enabled": "true", "id": "my_realm_1", "realm": "my_realm_1"})
       self.assertNotEqual(state, True)

    @classmethod
    def setUpClass(self):
        self.test_realm = "realm_for_testing"
        self.admin = KeycloakAdmin({
            "username": "6OPtWY33", 
            "password": "I6gglTeDLlmmpLYoAAUMcFQqNOMjw5dA"
            }, test_url)

        self.master_realm = self.admin.master()
        self.master_realm.create({"enabled": "true", "id": self.test_realm, "realm": self.test_realm})

        time.sleep(1)

     

    @classmethod
    def tearDownClass(self):
        realm = self.admin.master()
        remove_state = self.master_realm.remove(self.test_realm)
        

if __name__ == '__main__':
    unittest.main()



from .crud import KeycloakCRUD
from .targets import Targets
from .url import RestURL
import requests, json


def AuthenticationFlowURLBuilder(url):
    return Targets.makeWithURL(url).addResources(['flows'])

# The keycloak API is a bit crazy here they add with: 
# Post /parentId/executions/execution 
# 
# But they delete with: 
#
# DELETE /executions/<id>
#
# Sadly we need to customize the URL's in order to make it work.
#

def BuildAction(kcCRUD, parentFlow, actionType):
    parentFlowAlias = parentFlow['alias']
    kcCRUD.targets.addResourcesFor('create',[parentFlowAlias, 'executions', actionType])
    kcCRUD.targets.addResourcesFor('update',[parentFlowAlias, 'executions'])
    kcCRUD.targets.addResourcesFor('read',[parentFlowAlias, 'executions'])

    kcCRUD.targets.getDeleteMethod().replaceResource('flows', 'executions')

    return kcCRUD


class AuthenticationFlows(KeycloakCRUD):
    def setURL(self, url):
        super().setURL(url)
        self.targets.addResources(['flows'])


    def _load(self, token, targets): 
        flow = KeycloakCRUD()    
        flow.token = token 
        flow.targets = targets.copy()
        return flow

    # Generate a CRUD object pointing to /realm/<realm>/authentication/flow_alias/executions/flow
    def flows(self, authFlow):
        flow = self._load(self.token, self.targets)

        return BuildAction( 
                kcCRUD=flow, 
                parentFlow=authFlow,
                actionType='flow')

    # Generate a CRUD object pointing to /realm/<realm>/authentication/flow_alias/executions/execution
    def executions(self, execution):
        flow = self._load(self.token, self.targets)

        return BuildAction( 
                kcCRUD=flow, 
                parentFlow=execution,
                actionType='execution')

           








import unittest, time
from kcapi import OpenID, RestURL
from .testbed import TestBed 

WRONG_URL =  'https://sso-wrong-cvaldezr-stage.apps.sandbox-m2.ll9k.p1.openshiftapps.com'

class Testing_OpenID(unittest.TestCase):
    def test_oidc_ctor(self): 
        try:
            OpenID({"client_id":"admin-cli"}, WRONG_URL)
        except Exception as E:
            self.assertEqual("password" in str(E), True)

        try:
            OpenID({"client_id":"admin-cli", "realm": "my_realm", "password": "xxx"}, WRONG_URL)
        except Exception as E:
            self.assertEqual("username" in str(E), True)

        self.assertRaises(Exception, lambda: OpenID({"client_id":"admin-cli"})) 

    def test_oidc_login_on_wrong_url(self):
        oid_client = OpenID({
            "client_id": "admin-cli",
            "username": "6OPtWY33", 
            "password":"I6gglTeDLlmmpLYoAAUMcFQqNOMjw5dA", 
            "grant_type":"password",
            "realm" : "master"
            }, )
        
        self.assertRaises(Exception, lambda: oid_client.getToken()) 

    def test_oidc_login_on_wrong_password(self):
        oid_client = OpenID({
            "client_id": "admin-cli",
            "username": "6OPtWY33", 
            "password":"I6gglTeDLlmmpLYoAAUMcFQqNOMjw5dA", 
            "grant_type":"password",
            "realm" : "master"
            }, self.ENDPOINT)

        try:
            oid_client.getToken()
        except Exception as E:
            self.assertEqual("Unauthorized" in str(E), True)

    def test_creating_oidc_client_using_factory(self):
        oidc = OpenID.createAdminClient(self.USER, self.PASSWORD)
        self.assertIsNotNone( oidc.getToken(self.ENDPOINT) )

    def test_oidc_login(self):
        oid_client = OpenID({
            "client_id": "admin-cli",
            "username": self.testbed.USER, 
            "password": self.testbed.PASSWORD, 
            "grant_type":"password",
            "realm" : "master"
            }, self.ENDPOINT)

        token = oid_client.getToken()
        self.assertIsNotNone(token)
        self.assertTrue(len(token) > 0)
        
    @classmethod
    def setUpClass(self):
        self.testbed = TestBed()
        self.kc = self.testbed.getKeycloak()
        self.realm = self.testbed.REALM 
        self.master_realm = self.testbed.getAdminRealm()
        self.ENDPOINT = self.testbed.ENDPOINT
        self.USER = self.testbed.USER
        self.PASSWORD = self.testbed.PASSWORD
        
    @classmethod
    def tearDownClass(self):
        self.testbed.goodBye()



if __name__ == '__main__':
    unittest.main()



import requests, json
from .rest import RestURL


class OpenID:
    def __check_params(self, params):
        expected_params = ['password', 'username', 'grant_type', 'client_id']

        for param in expected_params:
            if param not in params:
                raise Exception("Missing parameter on OpenId class: ", param)

    @staticmethod
    def __raise_error(resp, url):
        code = resp.status_code

        if code in [404]:
            raise Exception("Server Error: " + str(code), resp.text, " URL: ", str(url))

        if code in [503, 500]:
            raise Exception("Server Error: " + str(code), " URL: ", str(url))

        if code == 401:
            raise Exception("Server returned 401: Unauthorized. Please check username or password.")

        json_data = resp.json()
        error_message = json_data["error"] + "--" + json_data["error_description"]
        raise Exception("Error: " + str(code) + " \n for URL:" + str(url) + " \n Response: " + error_message)

    # Retrieves the Well Known Endpoint: https://openid.net/specs/openid-connect-discovery-1_0.html
    @staticmethod
    def discover(url, realm):
        discovery_url = RestURL(url, ['auth', 'realms', realm, '.well-known', 'openid-configuration'])

        resp = requests.get(url=str(discovery_url))

        if resp.status_code == 200:
            return resp.json()

        OpenID.__raise_error(resp, discovery_url)

    def __init__(self, credentials, url=None):
        self.__check_params(credentials)

        self.credentials = credentials
        self.realm = self.credentials['realm']
        self.token = None
        self.urlObject = None

        if url:
            self.urlObject = url

    @staticmethod
    def createAdminClient(username, password):
        __props = {
            "client_id": "admin-cli",
            "grant_type": "password",
            "realm": "master",
            "username": username,
            "password": password
        }

        return OpenID(__props)

    def getToken(self, target_url=None):
        if not target_url and not self.urlObject:
            raise Exception('URL Not Found: Make sure you provide a URL before invoking the service')
        url = target_url if target_url else self.urlObject

        token_endpoint = OpenID.discover(url, self.realm)['token_endpoint']

        resp = requests.post(token_endpoint, data=self.credentials)

        if resp.status_code == 200:
            self.token = resp.json()["access_token"]
            return self.token
        else:
            OpenID.__raise_error(resp, url)



from .rest import KCResourceBuilder


class Keycloak:
    def __init__(self, token, url):
        if not token:
            raise Exception("No authentication token provided.")
        if not url:
            raise Exception("No Keycloak endpoint URL has been provided.")

        self.token = token
        self.url = url

    def admin(self):
        return KCResourceBuilder(self.url).build(self.token)

    def build(self, resourceName, realm):
        return KCResourceBuilder(self.url).withName(resourceName).forRealm(realm).build(self.token)
